/*
Copyright (c) 2024 Dusk
All rights reserved.

This code is proprietary to Dusk AI Inc.
The code may be used solely for accessing the Service provided by Dusk AI Inc. following the Dusk Terms of Service ("Terms") accessible at dusk.gg/eula. You may not use this code for any use or purpose other than as expressly permitted by the Terms.
Restrictions set forth in the Terms include, but is not limited to, that you may not copy, adapt, modify, prepare derivative works based upon, distribute, license, sell, transfer, publicly display, publicly perform, transmit, stream, broadcast, attempt to discover any source code, reverse engineer, decompile, dissemble, or otherwise exploit the code as a whole or any portion of the code.
*/
type UntypedGameState = Record<string, any>;
type UntypedInitLogicAction = (params?: any) => void;
type UntypedInitLogicActions = Record<string, UntypedInitLogicAction>;
type GameConfig = {
    minPlayers: number;
    maxPlayers: number;
    updatesPerSecond: number;
    eventCallbacks: {
        playerLeft: boolean;
        playerJoined: boolean;
    };
    update: boolean;
};
type SessionId = string;
type GameId = number;
type PlayerId = string;
type RandomSeed = number;
type Player = {
    playerId: PlayerId;
    displayName: string;
    avatarUrl: string;
};
type Players = Record<PlayerId, Player>;
type UserId = number;
type Spectator = {
    playerId: undefined;
};
type User = (Player | Spectator) & {
    userId: number;
    supportsDictionaryCompression: boolean;
};
type Users = Record<UserId, User>;
type Environment = "runtime" | "devUI" | "test";

type LogFn = {
    (obj: object, msg: string): void;
    (msg: string): void;
};
type MsgLogger = {
    error: LogFn;
    warn: LogFn;
    info: LogFn;
};

type ServerErrorMessages = keyof typeof errorMap;
declare const errorMap: {
    GAME_OVER_INVALID_OPTIONS: {
        devUIMessage: string;
        consoleMessage: (data: any) => string;
        printExtraDataInDevUI: true;
    };
    STATE_SYNC_TOO_BIG: {
        devUIMessage: (data: any) => string;
    };
    SERVER_UPDATE_LOOP_FAILED: {
        devUIMessage: string;
        printExtraDataInDevUI: true;
    };
    ACTION_FAILED: {
        devUIMessage: (data: any) => string;
        printExtraDataInDevUI: true;
    };
    PLAYER_JOINED_FAILED: {
        devUIMessage: string;
        printExtraDataInDevUI: true;
    };
    PLAYER_LEFT_FAILED: {
        devUIMessage: string;
        printExtraDataInDevUI: true;
    };
    UPDATE_LOOP_BEHIND_GAME_TIME: {
        devUIUnexpected: true;
    };
    SERVER_UPDATE_LOOP_FASTER_THAN_GAME_TIME: {
        devUIUnexpected: true;
    };
    GET_GAME_STATE_FAILED: {
        devUIUnexpected: true;
    };
    SERVER_RECEIVED_MESSAGE_TOO_BIG: {
        devUIUnexpected: true;
    };
    SERVER_MESSAGE_TO_CLIENTS_TOO_BIG: {
        devUIUnexpected: true;
    };
    SERVER_FULL: {
        devUIUnexpected: true;
    };
    ON_PLAYER_JOINED_CALLBACK_MISSING: {
        devUIUnexpected: true;
    };
    ACTION_FAILED_ROOM_GAME_STATE_MISSING: {
        devUIUnexpected: true;
    };
    PLAYER_JOINED_ROOM_GAME_STATE_MISSING: {
        devUIUnexpected: true;
    };
    PLAYER_LEFT_ROOM_GAME_STATE_MISSING: {
        devUIUnexpected: true;
    };
    ACTION_PARAMS_SIZE_OVER_LIMIT: {
        devUIUnexpected: true;
    };
};

type GameOverOptions = {
    players: {
        [playerId: PlayerId]: "WON" | "LOST" | number;
    };
    delayPopUp?: boolean;
};
type GameOverGameEnded = {
    reason: "gameEnded";
    options?: GameOverOptions;
};
type GameOverPlayerLeft = {
    reason: "playerLeft";
};
type GameOverMinPlayers = {
    reason: "minPlayers";
};
type GameOverError = {
    reason: "err";
    err?: {
        message: ServerErrorMessages;
        data?: object;
    };
};
type GameOverContext = (GameOverMinPlayers | GameOverPlayerLeft | GameOverGameEnded | GameOverError) & {
    players: Players;
};

type ContextWithGameState<Context, GameState> = Context & {
    game: GameState;
};
type EventContext = {
    allPlayerIds: PlayerId[];
};
type UpdateContext = {
    allPlayerIds: PlayerId[];
};
type ActionContext = {
    playerId: PlayerId;
    allPlayerIds: PlayerId[];
};
type InitLogicEvent<GameState> = (playerId: PlayerId, eventContext: ContextWithGameState<EventContext, GameState>) => void;
type InitLogicEvents<GameState> = {
    playerJoined?: InitLogicEvent<GameState>;
    playerLeft?: InitLogicEvent<GameState>;
};
type InitLogicUpdate<GameState> = (updateContext: ContextWithGameState<UpdateContext, GameState>) => void;
type InitLogicActions<GameState, GameActions extends UntypedInitLogicActions> = {
    [key in keyof GameActions]: (params: Parameters<GameActions[key]>[0], actionContext: ContextWithGameState<ActionContext, GameState>) => void;
};
type InitLogicParams<GameState, GameActions extends UntypedInitLogicActions> = {
    minPlayers: number;
    maxPlayers: number;
    updatesPerSecond?: number;
    setup: (allPlayerIds: PlayerId[]) => GameState;
    actions: InitLogicActions<GameState, GameActions>;
    events?: InitLogicEvents<GameState>;
    update?: InitLogicUpdate<GameState>;
    inputDelay?: number;
};
type SharedSdk<GameState, GameActions extends UntypedInitLogicActions> = {
    initLogic: (params: InitLogicParams<GameState, GameActions>) => void;
    invalidAction: () => Error;
    gameOver: (options?: GameOverOptions) => void;
    gameTimeInSeconds: () => number;
    gameTime: () => number;
};

type LogicRunnerContext = {
    randomSeed: RandomSeed;
    msPerTick: number;
    logicTick: number;
};
type LogicRunnerResult<T> = {
    error: "ROOM_GAME_STATE_MISSING";
} | {
    data: T;
};
type LogicRunnerGameContext = {
    gameOver: GameOverContext | null;
};
type LogicRunnerReturnContext = {
    gameContext: LogicRunnerGameContext;
    stateHash?: number;
};
type LogicRunnerAction = (roomId: number, logicContext: LogicRunnerContext, params: any, actionContext: ActionContext, calculateStateHash: boolean) => LogicRunnerResult<LogicRunnerReturnContext | false>;
type LogicRunnerActions = Record<string, LogicRunnerAction>;
type LogicRunnerEvent = (roomId: number, logicContext: LogicRunnerContext, playerId: PlayerId, eventContext: EventContext, calculateStateHash: boolean) => LogicRunnerResult<LogicRunnerReturnContext>;
type LogicRunnerEvents = {
    playerJoined?: LogicRunnerEvent;
    playerLeft?: LogicRunnerEvent;
};
type LogicRunnerUpdate = (roomId: number, logicContext: LogicRunnerContext, updateContext: UpdateContext) => LogicRunnerResult<LogicRunnerReturnContext>;
type LogicRunnerSetup = (roomId: number, logicContext: LogicRunnerContext, playerIds: PlayerId[], calculateStateHash: boolean) => {
    gameState: UntypedGameState;
    stateHash?: number;
};
type SdkServer = {
    gameConfig?: GameConfig;
    setup?: LogicRunnerSetup;
    actions?: LogicRunnerActions;
    events?: LogicRunnerEvents;
    update?: LogicRunnerUpdate;
    logicContext: LogicRunnerContext;
    gameOverContext: GameOverContext | null;
    rehydrate: (roomId: number, gameState: UntypedGameState) => void;
    getActionNames: () => string[];
    getGameState: (roomId: number, calculateStateHash: boolean) => LogicRunnerResult<{
        gameState: UntypedGameState;
        stateHash?: number;
    }>;
    cleanup: (roomId: number) => void;
    runAction: (action: string, ...args: Parameters<LogicRunnerAction>) => ReturnType<LogicRunnerAction>;
    runEvent: (event: "playerJoined" | "playerLeft", ...args: Parameters<LogicRunnerEvent>) => ReturnType<LogicRunnerEvent>;
} & SharedSdk<UntypedGameState, UntypedInitLogicActions>;

type TimelineEntryStart = {
    type: "START";
    serverSeed: number;
    initialPlayers: Players;
    stateHash: number;
};
type TimelineEntryEnd = {
    type: "END";
    gameState: UntypedGameState | string;
    stateHash: number;
    reason: GameOverContext["reason"] | "stop";
};
type TimelineEntryWithSuccess<T> = T & ({
    success: false;
} | {
    success: true;
    stateHash: number;
});
type TimelineEntryWithoutOrder = {
    logicTick: number;
} & (TimelineEntryStart | TimelineEntryEnd | TimelineEntryWithSuccess<{
    type: "ACTION";
    playerId: PlayerId;
    action: string;
    params: any;
    randomSeed: number;
}> | TimelineEntryWithSuccess<{
    type: "PLAYER_JOINED";
    player: Player;
}> | TimelineEntryWithSuccess<{
    type: "PLAYER_LEFT";
    playerId: PlayerId;
}>);
type TimelineEntry = TimelineEntryWithoutOrder & {
    order: number;
};

type PlayersRandomState = Record<PlayerId, {
    seed: RandomSeed;
    actionCount: number;
}>;
type GameContext = {
    readonly gameOver: GameOverContext | null;
    orderNumber: number;
    sessionId: SessionId;
    gameId: GameId;
};
type ServerSerializationData<GameState> = {
    game: GameState;
} & ServerState;
type ServerState = {
    context: GameContext;
    random: PlayersRandomState;
    gameTime: number;
    updateCount: number;
};
type UpdateLoopContext = {
    logicTick: number;
};
type RunGameLogic = {
    getConfig: () => GameConfig;
    getActionNames: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["getActionNames"]>) => Promise<ReturnType<SdkServer["getActionNames"]>>;
    getGameState: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["getGameState"]>) => Promise<ReturnType<SdkServer["getGameState"]>>;
    runSetup: (msgLogger: MsgLogger, ...args: Parameters<LogicRunnerSetup>) => Promise<ReturnType<LogicRunnerSetup>>;
    runEvent: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["runEvent"]>) => Promise<ReturnType<SdkServer["runEvent"]>>;
    runAction: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["runAction"]>) => Promise<ReturnType<SdkServer["runAction"]>>;
    runUpdate: (msgLogger: MsgLogger, ...args: Parameters<LogicRunnerUpdate>) => Promise<ReturnType<LogicRunnerUpdate>>;
    rehydrate: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["rehydrate"]>) => Promise<ReturnType<SdkServer["rehydrate"]>>;
    cleanup: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["cleanup"]>) => Promise<ReturnType<SdkServer["cleanup"]>>;
};
type GameServer<GameState> = {
    onMsg: (msgLogger: MsgLogger, userId: UserId, msg: string, info: {
        serverPreprocessingDurationInMs: number;
    }) => Promise<void>;
    onPlayerJoined: (msgLogger: MsgLogger, playerId: PlayerId) => Promise<void>;
    onPlayerLeft: (msgLogger: MsgLogger, playerId: PlayerId) => Promise<void>;
    getDataForSerialization: (msgLogger: MsgLogger) => Promise<ServerSerializationData<GameState>>;
    getConfig: () => GameConfig;
    getSessionId: () => SessionId;
    getGameId: () => GameId;
    isGameOver: () => boolean;
    onUpdateLoop: (msgLogger: MsgLogger, updateLoopContext: UpdateLoopContext) => Promise<void>;
    cleanup: (msgLogger: MsgLogger) => Promise<void>;
};
type NetworkServer = {
    getUsers: () => Users;
    broadcastMsg: (msgLogger: MsgLogger, msg: string) => void;
    sendMsg: (msgLogger: MsgLogger, userId: UserId, msg: string) => void;
    queueUpdateLoop: (updateLoopContext: UpdateLoopContext) => void;
    onGameOver?: (msgLogger: MsgLogger, reason: GameOverContext["reason"]) => void;
    onTimelineEntry?: (msgLogger: MsgLogger, timelineEntry: TimelineEntry) => void;
    getSeedForPlayerId?: (playerId: PlayerId) => number;
};
type CreateGameServerProps = {
    initialMsgLogger: MsgLogger;
    network: NetworkServer;
    logic: RunGameLogic;
    flags: {
        logServerClientGameTime?: boolean;
        logNetworkMsg?: boolean;
        logWrongTick?: boolean;
        logNetworkMsgSize?: boolean;
        sendInitialStateSync?: boolean;
        sendStateHash?: boolean;
        enableDictionaryCompression?: boolean;
    };
    params: {
        sessionId: string | null;
        roomId: number;
        gameId: GameId;
        serverSeed: RandomSeed;
        initialServerData?: ServerSerializationData<UntypedGameState>;
    };
    config: {
        logicTimeoutDuration?: number | false;
        allowedGameTimeDelay?: number;
        gameTimeUpdateEveryXTicks?: number;
        environment: Environment;
    };
};

declare function generateId(length?: number): string;

declare function isNewUserSpectator(users: Users, gameConfig: GameConfig, isGameOver: boolean): boolean;
declare function canSwitchSpectatorToPlayer(users: Users, gameConfig: GameConfig): boolean;

declare function createSdkForLogicRunner(globalObject?: typeof globalThis): SdkServer;

declare function createGameServer({ initialMsgLogger, params: { gameId, roomId, sessionId, initialServerData, serverSeed }, flags: { sendStateHash: sendStateHashFlag, logWrongTick, logNetworkMsg, logNetworkMsgSize, sendInitialStateSync, enableDictionaryCompression, }, config: { logicTimeoutDuration, allowedGameTimeDelay, gameTimeUpdateEveryXTicks, environment, }, network, logic, }: CreateGameServerProps): Promise<GameServer<UntypedGameState>>;

export { CreateGameServerProps, GameConfig, GameServer, RunGameLogic, SdkServer, ServerSerializationData, canSwitchSpectatorToPlayer, createGameServer, createSdkForLogicRunner, generateId, isNewUserSpectator };
