/*
Copyright (c) 2024 Dusk
All rights reserved.

This code is proprietary to Dusk AI Inc.
The code may be used solely for accessing the Service provided by Dusk AI Inc. following the Dusk Terms of Service ("Terms") accessible at dusk.gg/eula. You may not use this code for any use or purpose other than as expressly permitted by the Terms.
Restrictions set forth in the Terms include, but is not limited to, that you may not copy, adapt, modify, prepare derivative works based upon, distribute, license, sell, transfer, publicly display, publicly perform, transmit, stream, broadcast, attempt to discover any source code, reverse engineer, decompile, dissemble, or otherwise exploit the code as a whole or any portion of the code.
*/
type UntypedGameState = Record<string, any>;
type UntypedInitLogicAction = (params?: any) => void;
type UntypedInitLogicActions = Record<string, UntypedInitLogicAction>;
type GameConfig = {
    minPlayers: number;
    maxPlayers: number;
    updatesPerSecond: number;
    eventCallbacks: {
        playerLeft: boolean;
        playerJoined: boolean;
    };
    update: boolean;
};
type SessionId = string;
type GameId = number;
type PlayerId = string;
type RandomSeed = number;
type Player = {
    playerId: PlayerId;
    displayName: string;
    avatarUrl: string;
};
type Players = Record<PlayerId, Player>;
type UserId = number;
type Spectator = {
    playerId: undefined;
};
type User = (Player | Spectator) & {
    userId: number;
    supportsDictionaryCompression: boolean;
};
type Users = Record<UserId, User>;
type FutureRequiredGameId = number | null;
type Environment = "runtime" | "devUI" | "test";

type LogFn = {
    (obj: object, msg: string): void;
    (msg: string): void;
};
type MsgLogger = {
    error: LogFn;
    warn: LogFn;
    info: LogFn;
};

type ServerErrorMessages = keyof typeof errorMap;
declare const errorMap: {
    GAME_OVER_INVALID_OPTIONS: {
        devUIMessage: string;
        consoleMessage: (data: any) => string;
        printExtraDataInDevUI: true;
    };
    STATE_SYNC_TOO_BIG: {
        devUIMessage: (data: any) => string;
    };
    SERVER_UPDATE_LOOP_FAILED: {
        devUIMessage: string;
        printExtraDataInDevUI: true;
    };
    ACTION_FAILED: {
        devUIMessage: (data: any) => string;
        printExtraDataInDevUI: true;
    };
    PLAYER_JOINED_FAILED: {
        devUIMessage: string;
        printExtraDataInDevUI: true;
    };
    PLAYER_LEFT_FAILED: {
        devUIMessage: string;
        printExtraDataInDevUI: true;
    };
    UPDATE_LOOP_BEHIND_GAME_TIME: {
        devUIUnexpected: true;
    };
    SERVER_UPDATE_LOOP_FASTER_THAN_GAME_TIME: {
        devUIUnexpected: true;
    };
    GET_GAME_STATE_FAILED: {
        devUIUnexpected: true;
    };
    SERVER_RECEIVED_MESSAGE_TOO_BIG: {
        devUIUnexpected: true;
    };
    SERVER_MESSAGE_TO_CLIENTS_TOO_BIG: {
        devUIUnexpected: true;
    };
    SERVER_FULL: {
        devUIUnexpected: true;
    };
    ON_PLAYER_JOINED_CALLBACK_MISSING: {
        devUIUnexpected: true;
    };
    ACTION_FAILED_ROOM_GAME_STATE_MISSING: {
        devUIUnexpected: true;
    };
    PLAYER_JOINED_ROOM_GAME_STATE_MISSING: {
        devUIUnexpected: true;
    };
    PLAYER_LEFT_ROOM_GAME_STATE_MISSING: {
        devUIUnexpected: true;
    };
    ACTION_PARAMS_SIZE_OVER_LIMIT: {
        devUIUnexpected: true;
    };
};

type GameOverOptions = {
    players: {
        [playerId: PlayerId]: "WON" | "LOST" | number;
    };
    delayPopUp?: boolean;
};
type GameOverGameEnded = {
    reason: "gameEnded";
    options?: GameOverOptions;
};
type GameOverPlayerLeft = {
    reason: "playerLeft";
};
type GameOverMinPlayers = {
    reason: "minPlayers";
};
type GameOverError = {
    reason: "err";
    err?: {
        message: ServerErrorMessages;
        data?: object;
    };
};
type GameOverContext = (GameOverMinPlayers | GameOverPlayerLeft | GameOverGameEnded | GameOverError) & {
    players: Players;
};

type ContextWithGameState<Context, GameState> = Context & {
    game: GameState;
};
type EventContext = {
    allPlayerIds: PlayerId[];
};
type UpdateContext = {
    allPlayerIds: PlayerId[];
};
type ActionContext = {
    playerId: PlayerId;
    allPlayerIds: PlayerId[];
};
type InitLogicEvent<GameState> = (playerId: PlayerId, eventContext: ContextWithGameState<EventContext, GameState>) => void;
type InitLogicEvents<GameState> = {
    playerJoined?: InitLogicEvent<GameState>;
    playerLeft?: InitLogicEvent<GameState>;
};
type InitLogicUpdate<GameState> = (updateContext: ContextWithGameState<UpdateContext, GameState>) => void;
type InitLogicActions<GameState, GameActions extends UntypedInitLogicActions> = {
    [key in keyof GameActions]: (params: Parameters<GameActions[key]>[0], actionContext: ContextWithGameState<ActionContext, GameState>) => void;
};
type InitLogicParams<GameState, GameActions extends UntypedInitLogicActions> = {
    minPlayers: number;
    maxPlayers: number;
    updatesPerSecond?: number;
    setup: (allPlayerIds: PlayerId[]) => GameState;
    actions: InitLogicActions<GameState, GameActions>;
    events?: InitLogicEvents<GameState>;
    update?: InitLogicUpdate<GameState>;
    inputDelay?: number;
};
type SharedSdk<GameState, GameActions extends UntypedInitLogicActions> = {
    initLogic: (params: InitLogicParams<GameState, GameActions>) => void;
    invalidAction: () => Error;
    gameOver: (options?: GameOverOptions) => void;
    gameTimeInSeconds: () => number;
    gameTime: () => number;
};

type LogicRunnerContext = {
    randomSeed: RandomSeed;
    msPerTick: number;
    logicTick: number;
};
type LogicRunnerResult<T> = {
    error: "ROOM_GAME_STATE_MISSING";
} | {
    data: T;
};
type LogicRunnerGameContext = {
    gameOver: GameOverContext | null;
};
type LogicRunnerReturnContext = {
    gameContext: LogicRunnerGameContext;
    stateHash?: number;
};
type LogicRunnerAction = (roomId: number, logicContext: LogicRunnerContext, params: any, actionContext: ActionContext, calculateStateHash: boolean) => LogicRunnerResult<LogicRunnerReturnContext | false>;
type LogicRunnerActions = Record<string, LogicRunnerAction>;
type LogicRunnerEvent = (roomId: number, logicContext: LogicRunnerContext, playerId: PlayerId, eventContext: EventContext, calculateStateHash: boolean) => LogicRunnerResult<LogicRunnerReturnContext>;
type LogicRunnerEvents = {
    playerJoined?: LogicRunnerEvent;
    playerLeft?: LogicRunnerEvent;
};
type LogicRunnerUpdate = (roomId: number, logicContext: LogicRunnerContext, updateContext: UpdateContext) => LogicRunnerResult<LogicRunnerReturnContext>;
type LogicRunnerSetup = (roomId: number, logicContext: LogicRunnerContext, playerIds: PlayerId[], calculateStateHash: boolean) => {
    gameState: UntypedGameState;
    stateHash?: number;
};
type SdkServer = {
    gameConfig?: GameConfig;
    setup?: LogicRunnerSetup;
    actions?: LogicRunnerActions;
    events?: LogicRunnerEvents;
    update?: LogicRunnerUpdate;
    logicContext: LogicRunnerContext;
    gameOverContext: GameOverContext | null;
    rehydrate: (roomId: number, gameState: UntypedGameState) => void;
    getActionNames: () => string[];
    getGameState: (roomId: number, calculateStateHash: boolean) => LogicRunnerResult<{
        gameState: UntypedGameState;
        stateHash?: number;
    }>;
    cleanup: (roomId: number) => void;
    runAction: (action: string, ...args: Parameters<LogicRunnerAction>) => ReturnType<LogicRunnerAction>;
    runEvent: (event: "playerJoined" | "playerLeft", ...args: Parameters<LogicRunnerEvent>) => ReturnType<LogicRunnerEvent>;
} & SharedSdk<UntypedGameState, UntypedInitLogicActions>;

type TimelineEntryStart = {
    type: "START";
    serverSeed: number;
    initialPlayers: Players;
    stateHash: number;
};
type TimelineEntryEnd = {
    type: "END";
    gameState: UntypedGameState | string;
    stateHash: number;
    reason: GameOverContext["reason"] | "stop";
};
type TimelineEntryWithSuccess<T> = T & ({
    success: false;
} | {
    success: true;
    stateHash: number;
});
type TimelineEntryWithoutOrder = {
    logicTick: number;
} & (TimelineEntryStart | TimelineEntryEnd | TimelineEntryWithSuccess<{
    type: "ACTION";
    playerId: PlayerId;
    action: string;
    params: any;
    randomSeed: number;
}> | TimelineEntryWithSuccess<{
    type: "PLAYER_JOINED";
    player: Player;
}> | TimelineEntryWithSuccess<{
    type: "PLAYER_LEFT";
    playerId: PlayerId;
}>);
type TimelineEntry = TimelineEntryWithoutOrder & {
    order: number;
};

type PlayersRandomState = Record<PlayerId, {
    seed: RandomSeed;
    actionCount: number;
}>;
type GameContext = {
    readonly gameOver: GameOverContext | null;
    orderNumber: number;
    sessionId: SessionId;
    gameId: GameId;
};
type ServerSerializationData<GameState> = {
    game: GameState;
} & ServerState;
type ServerState = {
    context: GameContext;
    random: PlayersRandomState;
    gameTime: number;
    updateCount: number;
};
type UpdateLoopContext = {
    logicTick: number;
};
type RunGameLogic = {
    getConfig: () => GameConfig;
    getActionNames: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["getActionNames"]>) => Promise<ReturnType<SdkServer["getActionNames"]>>;
    getGameState: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["getGameState"]>) => Promise<ReturnType<SdkServer["getGameState"]>>;
    runSetup: (msgLogger: MsgLogger, ...args: Parameters<LogicRunnerSetup>) => Promise<ReturnType<LogicRunnerSetup>>;
    runEvent: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["runEvent"]>) => Promise<ReturnType<SdkServer["runEvent"]>>;
    runAction: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["runAction"]>) => Promise<ReturnType<SdkServer["runAction"]>>;
    runUpdate: (msgLogger: MsgLogger, ...args: Parameters<LogicRunnerUpdate>) => Promise<ReturnType<LogicRunnerUpdate>>;
    rehydrate: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["rehydrate"]>) => Promise<ReturnType<SdkServer["rehydrate"]>>;
    cleanup: (msgLogger: MsgLogger, ...args: Parameters<SdkServer["cleanup"]>) => Promise<ReturnType<SdkServer["cleanup"]>>;
};
type GameServer<GameState> = {
    onMsg: (msgLogger: MsgLogger, userId: UserId, msg: string, info: {
        serverPreprocessingDurationInMs: number;
    }) => Promise<void>;
    onPlayerJoined: (msgLogger: MsgLogger, playerId: PlayerId) => Promise<void>;
    onPlayerLeft: (msgLogger: MsgLogger, playerId: PlayerId) => Promise<void>;
    getDataForSerialization: (msgLogger: MsgLogger) => Promise<ServerSerializationData<GameState>>;
    getConfig: () => GameConfig;
    getSessionId: () => SessionId;
    getGameId: () => GameId;
    isGameOver: () => boolean;
    onUpdateLoop: (msgLogger: MsgLogger, updateLoopContext: UpdateLoopContext) => Promise<void>;
    cleanup: (msgLogger: MsgLogger) => Promise<void>;
};
type NetworkServer = {
    getUsers: () => Users;
    broadcastMsg: (msgLogger: MsgLogger, msg: string) => void;
    sendMsg: (msgLogger: MsgLogger, userId: UserId, msg: string) => void;
    queueUpdateLoop: (updateLoopContext: UpdateLoopContext) => void;
    onGameOver?: (msgLogger: MsgLogger, reason: GameOverContext["reason"]) => void;
    onTimelineEntry?: (msgLogger: MsgLogger, timelineEntry: TimelineEntry) => void;
    getSeedForPlayerId?: (playerId: PlayerId) => number;
};
type CreateGameServerProps = {
    initialMsgLogger: MsgLogger;
    network: NetworkServer;
    logic: RunGameLogic;
    flags: {
        logServerClientGameTime?: boolean;
        logNetworkMsg?: boolean;
        logWrongTick?: boolean;
        logNetworkMsgSize?: boolean;
        sendInitialStateSync?: boolean;
        sendStateHash?: boolean;
        enableDictionaryCompression?: boolean;
    };
    params: {
        sessionId: string | null;
        roomId: number;
        gameId: GameId;
        serverSeed: RandomSeed;
        initialServerData?: ServerSerializationData<UntypedGameState>;
    };
    config: {
        logicTimeoutDuration?: number | false;
        allowedGameTimeDelay?: number;
        gameTimeUpdateEveryXTicks?: number;
        environment: Environment;
    };
};

type ServerToGameBase = {
    orderNumber: number;
    serverGameTime: number;
    logicTick: number;
};
type OnChangeAction<GameActions extends UntypedInitLogicActions> = {
    [Key in keyof GameActions]: {
        action?: Key;
        name: Key;
        playerId: PlayerId;
        params: Parameters<GameActions[Key]>[0];
    };
}[keyof GameActions];
type OnChangeMsgAction<GameActions extends UntypedInitLogicActions> = {
    [Key in keyof GameActions]: {
        action: Key;
        playerId: PlayerId;
        params: Parameters<GameActions[Key]>[0];
    };
}[keyof GameActions];
type ActionBase = {
    uuid: string;
    actionCount: number;
    sessionId: SessionId;
    randomSeed: RandomSeed;
    measureLatencyStart: number;
};
type GameToServerAction<GameActions extends UntypedInitLogicActions> = OnChangeMsgAction<GameActions> & ActionBase & {
    clientGameTime: number;
    expectedLogicTick: number;
};
type ServerToGameAction<GameActions extends UntypedInitLogicActions> = ServerToGameBase & OnChangeMsgAction<GameActions> & ActionBase & {
    stateHash: number | undefined;
};
type OnChangeStateSyncEvent = {
    event: "stateSync";
};
type ServerToGameGameTimeUpdate = ServerToGameBase & {
    event: "gameTimeUpdate";
    params: {
        gameContext: GameContext;
    };
};
type ServerToGameLatencyEstimate = {
    event: "latencyEstimate";
    measureLatencyStart: number;
    serverGameTime: number;
};
type ServerToGameStopGameEvent = {
    event: "debugStopGame";
};
type ServerToGameStateSyncEvent<GameState> = ServerToGameBase & OnChangeStateSyncEvent & {
    params: {
        game: GameState;
        gameContext: GameContext;
        players: Players;
        pastPlayerIds: PlayerId[];
        yourPlayerSeed: RandomSeed | undefined;
        yourPlayerActionCount: number | undefined;
        yourPlayerId: PlayerId | undefined;
        serverSeed: RandomSeed;
        triggeredByUpdateLoop: boolean;
        measureLatencyStart?: number;
        latencyRttEstimate?: number;
    };
    dictionary?: string[];
};
type OnChangePlayerJoinedEvent = {
    event: "playerJoined";
    params: {
        playerId: PlayerId;
    };
};
type ServerToGamePlayerJoinedEvent = ServerToGameBase & OnChangePlayerJoinedEvent & {
    params: {
        playerId: PlayerId;
        gameContext: GameContext;
        players: Players;
        randomSeed: RandomSeed;
        stateHash: number | undefined;
    };
};
type OnChangePlayerLeftEvent = {
    event: "playerLeft";
    params: {
        playerId: PlayerId;
    };
};
type ServerToGamePlayerLeftEvent = ServerToGameBase & OnChangePlayerLeftEvent & {
    params: {
        playerId: PlayerId;
        gameContext: GameContext;
        players: Players;
        randomSeed: RandomSeed;
        stateHash: number | undefined;
    };
};
type OnChangeTimeSyncEvent = {
    event: "timeSync";
};
type OnChangeUpdateEvent = {
    event: "update";
};
type NameAndEvent<T, N> = Omit<T, "event"> & {
    name: N;
    event?: N;
};
type OnChangeEvent = NameAndEvent<OnChangeStateSyncEvent, "stateSync"> | NameAndEvent<OnChangePlayerJoinedEvent, "playerJoined"> | NameAndEvent<OnChangePlayerLeftEvent, "playerLeft"> | NameAndEvent<OnChangeTimeSyncEvent, "timeSync"> | NameAndEvent<OnChangeUpdateEvent, "update">;
type ServerToGameOnChangeTriggeringEvent<GameState> = ServerToGameStateSyncEvent<GameState> | ServerToGamePlayerJoinedEvent | ServerToGamePlayerLeftEvent;
type ServerToGameEvent<GameState> = ServerToGameOnChangeTriggeringEvent<GameState> | ServerToGameGameTimeUpdate | ServerToGameLatencyEstimate | ServerToGameStopGameEvent;
type ServerToGame<GameState, GameActions extends UntypedInitLogicActions> = ServerToGameAction<GameActions> | ServerToGameEvent<GameState>;
type GameToServer<GameActions extends UntypedInitLogicActions> = GameToServerAction<GameActions> | GameToServerStateSyncRequest | {
    type: "MEASURE_LATENCY";
    measureLatencyStart: number;
} | {
    type: "DEBUG_TIMELINE";
    data: any;
};
type GameToServerStateSyncRequest = {
    type: "REQUEST_STATE_SYNC";
    measureLatencyStart: number;
    gameId: number | null;
};
type GameToServerWithGameId<GameActions extends UntypedInitLogicActions> = GameToServer<GameActions> & {
    gameId: FutureRequiredGameId;
};

type ClientToGame = {
    type: "setForceMuteStatus";
    muted: boolean;
} | {
    type: "DISCONNECTED";
} | {
    type: "CLIPBOARD_WRITE_RES";
    reqId: string;
    success: boolean;
} | {
    type: "CLIPBOARD_READ_RES";
    reqId: string;
    text: string | null;
} | {
    type: "STORAGE_WRITE_RES";
    reqId: string;
    success: boolean;
} | {
    type: "STORAGE_READ_RES";
    reqId: string;
    value: string | null;
};
type GameToClient = {
    type: "INIT";
    version: string;
} | {
    type: "ERR";
    msg: string;
    err?: string;
    info: object;
} | {
    type: "WARNING";
    msg: string;
    data?: unknown;
} | {
    type: "INFO";
    msg: string;
    data?: unknown;
} | {
    type: "WINDOW_ERR";
    err: {
        msg: string;
        filename: string;
        lineno: number;
        colno: number;
    };
} | {
    type: "WINDOW_ALERT";
    message: string;
} | {
    type: "BROWSER_INITIAL_OVERLAY_CLICKED";
} | {
    type: "BROWSER_IFRAME_LOADED";
} | {
    type: "GAME_OVER";
    context: GameOverContext;
} | {
    type: "OPEN_EXTERNAL_APP";
    url: string;
} | {
    type: "CLIPBOARD_WRITE_REQ";
    reqId: string;
    text: string;
} | {
    type: "CLIPBOARD_READ_REQ";
    reqId: string;
} | {
    type: "STORAGE_WRITE_REQ";
    reqId: string;
    key: string;
    value: string;
} | {
    type: "STORAGE_READ_REQ";
    reqId: string;
    key: string;
} | {
    type: "SHOW_GAME_OVER_POP_UP";
} | {
    type: "SHOW_INVITE_SHEET";
} | {
    type: "ON_CHANGE";
} | {
    type: "FPS_STATS";
    min: number;
    max: number;
    avg: number;
} | {
    type: "FPS_WARNING";
    fps: number;
    isLowFps: boolean;
};
type GameToClientWithGameId = GameToClient & {
    gameId: FutureRequiredGameId;
};
type GameDevSdkNetworkLayer<GameState, GameActions extends UntypedInitLogicActions> = {
    sendGameToServer: (gameToServer: GameToServer<any>, options?: {
        dictionary?: string[];
    }) => void;
    sendGameToClient: (gameToClient: GameToClient) => void;
    onMsg: (callback: (msg: {
        clientToGame: ClientToGame;
    } | {
        serverToGame: ServerToGame<GameState, GameActions>;
    }) => void, getDictionary: () => string[]) => void;
};
type GameClient = {
    onServerMsg: (msg: string) => void;
    onGameMsg: (msg: string) => void;
    onDisconnected: () => void;
    onReconnected: () => void;
    onGameReloaded: () => void;
    sendMsgToGame: (msg: ClientToGame) => void;
    stop: () => void;
    _getCompressionDictionary: () => string[];
};

type InterpolatorFactory = <Dimensions extends number | number[]>() => Interpolator<Dimensions>;
type Interpolator<Dimensions extends number | number[]> = {
    update: (params: {
        game: Dimensions;
        futureGame: Dimensions;
    }) => void;
    getPosition: () => Dimensions;
};
type InterpolatorLatencyFactory = <Dimensions extends number | number[]>(config: {
    maxSpeed: number;
    timeToMaxSpeed?: number;
}) => InterpolatorLatency<Dimensions>;
type InterpolatorLatency<Dimensions extends number | number[]> = {
    update: (params: {
        game: Dimensions;
        futureGame: Dimensions;
    }) => void;
    getPosition: () => Dimensions;
    jump: (game: Dimensions) => void;
};

type NetworkClient = {
    sendMsgToServer: (msg: string) => void;
    sendMsgToGame: (msg: string) => void;
};
type ClientAction<GameActions extends UntypedInitLogicActions> = GameToServerAction<GameActions> & {
    clientLogicTick: number;
};
type OnChange<GameState, GameActions extends UntypedInitLogicActions> = (params: {
    game: GameState;
    action?: OnChangeAction<GameActions>;
    event?: OnChangeEvent;
    yourPlayerId: PlayerId | undefined;
    /** @deprecated Use allPlayerIds in combination with getPlayerInfo()  */
    players: Players;
    allPlayerIds: PlayerId[];
    rollbacks: OnChangeAction<GameActions>[];
    previousGame: GameState;
    futureGame?: GameState;
}) => void;
type Sdk<GameState, GameActions extends UntypedInitLogicActions> = {
    initClient: (params: {
        onChange: OnChange<GameState, GameActions>;
    }) => void;
    actions: GameActions;
    version: string;
    openExternalApp: (url: string) => void;
    clipboardWrite: (text: string) => Promise<{
        success: boolean;
    }>;
    clipboardRead: () => Promise<{
        text: string | null;
    }>;
    storageWrite: (key: string, value: string) => Promise<{
        success: boolean;
    }>;
    storageRead: (key: string) => Promise<{
        value: string | null;
    }>;
    showGameOverPopUp: () => void;
    showInvitePlayers: () => void;
    timeSinceLastUpdate: () => number;
    msPerUpdate: number;
    interpolator: InterpolatorFactory;
    interpolatorLatency: InterpolatorLatencyFactory;
    getPlayerInfo: (playerId: PlayerId) => Player;
} & SharedSdk<GameState, GameActions>;
type RuneClient<GameState, GameActions extends UntypedInitLogicActions> = Sdk<GameState, GameActions>;
type DuskClient<GameState, GameActions extends UntypedInitLogicActions> = Sdk<GameState, GameActions>;

type OnGameStateChangedStates = {
    state: "LOADING";
} | {
    state: "PLAYING";
} | {
    state: "ERR";
} | {
    state: "GAME_OVER";
    context: GameOverContext;
};

declare const sdk: Sdk<UntypedGameState, UntypedInitLogicActions>;

export { ActionContext, ClientAction, ClientToGame, ContextWithGameState, CreateGameServerProps, DuskClient, Environment, EventContext, FutureRequiredGameId, GameClient, GameConfig, GameContext, GameDevSdkNetworkLayer, GameId, GameOverContext, GameOverOptions, GameServer, GameToClient, GameToClientWithGameId, GameToServer, GameToServerAction, GameToServerStateSyncRequest, GameToServerWithGameId, InitLogicActions, InitLogicEvents, InitLogicParams, InitLogicUpdate, Interpolator, InterpolatorLatency, LogicRunnerContext, LogicRunnerGameContext, LogicRunnerResult, LogicRunnerReturnContext, LogicRunnerSetup, LogicRunnerUpdate, MsgLogger, NetworkClient, NetworkServer, OnChange, OnChangeAction, OnChangeEvent, OnChangePlayerJoinedEvent, OnChangePlayerLeftEvent, OnChangeStateSyncEvent, OnChangeTimeSyncEvent, OnChangeUpdateEvent, OnGameStateChangedStates, Player, PlayerId, Players, PlayersRandomState, RandomSeed, RunGameLogic, RuneClient, Sdk, SdkServer, ServerSerializationData, ServerState, ServerToGame, ServerToGameAction, ServerToGameEvent, ServerToGameGameTimeUpdate, ServerToGameLatencyEstimate, ServerToGameOnChangeTriggeringEvent, ServerToGamePlayerJoinedEvent, ServerToGamePlayerLeftEvent, ServerToGameStateSyncEvent, ServerToGameStopGameEvent, SessionId, SharedSdk, Spectator, UntypedGameState, UntypedInitLogicActions, UpdateContext, User, UserId, Users, sdk as default };
